#include "pamImage.h"
#include "dflBinarylib.h"
#include "bordertracinglib.h"
#include "dflContourlinefitlib.h"
#include "dflUtils.h"
#include "dflCOLDfeatlib.h"

#include <iostream>
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <cmath>

int main(int argc,char*argv[])
{
	if(argc!=6)
	{
		std::cout<<"Usage: "<<argv[0]<<" ppm label outfile model binary(IAM-0&Firemaker1)\n";
		return 1;
	}
	
	PamImage* im = new PamImage(argv[1]);
	std::string label(argv[2]);
	std::string outfile(argv[3]);
	int model=atoi(argv[4]);
	int binary = atof(argv[5]);

	
	if(im->getImageType()!=2)
		im=im->convert(2);
	
	PamImage* otsu;
	
	if(binary > 0)
	{
		dplBinaryMethod bin(*im,false);
		otsu=bin.run(0);
		otsu->save("binary.ppm");
	}
	else
		otsu = covert_ppm(im);

	const int w = im->getWidth();
	const int h = im->getHeight();
	
	COLDfeat cold;
	
	int kLength = 3;
	std::vector<float> feature = cold.getCOLDfeat(otsu,kLength);

	std::cout<<"get feature size: "<<feature.size()<<"\n";
	
	FILE* fp = fopen(outfile.c_str(),"w");
	fprintf(fp,"%s %d ",label.c_str(),(int)feature.size());
	
	for(int n = 0; n < feature.size(); ++n)
		fprintf(fp,"%.6f ",feature[n]);
	fprintf(fp,"\n");
	fclose(fp);
	
	delete im;
	delete otsu;
	return 1;
}

