/* This source file is a part of 
 * 
 * Document feature library (DFLib)
 * 
 * @copyright: Sheng He, University of Groningen (RUG)
 * 
 * Email: heshengxgd@gmail.com
 * 
 *  27, Nov., 2015
 * 
 * 
 *    Cloud Of Lines Discriptor for handwritten documents
 * */
#ifndef _DFL_COLDFEATLIB_H_
#define _DFL_COLDFEATLIB_H_

#include <vector>
#include <iostream>

#include "dflUtils.h"
#include "pamImage.h"
#include "bordertracinglib.h"
#include "dflContourlinefitlib.h"

/** keypointlist contains the dominant points of each contour 
 * @para scale is the maximum radius for log-polar space */
 
typedef struct{
	dflPixel point;
	int number;
}linevec;

class COLDfeat{
	public:
		COLDfeat(float _scale=5.0,int _nrad=7,int _nang=12);
		~COLDfeat();
		
		/** @para: kLength the $k$ in the paper
		 *         The final feature is formed by concatenating $k$-th COLDs *
		 *  @keypointlist: the dominant points on the coutrs */
		std::vector<float> getCOLDfeat(std::vector<std::vector<dflPixel> >& keypointlist,int kLength);
		
		/** @binary: the binary images */
		std::vector<float> getCOLDfeat(PamImage* binary, int kLength,float threshold=3.0);
		
		/** model == 1 compute the angle histogram
		 *  model == 2 compute the line histogram
		 *  model == 3 compute the concatenate of angle and line histogram
		 * */
		std::vector<float> getCOLDfeatLineAngleSeperately(PamImage* binary, int kLength,float threshold=3.0,int model=3);
		
	private:
		
		/** add k length lines */
		std::vector<linevec> getLineVector(std::vector<std::vector<dflPixel> >& keypointlist,int kLength=1);
		
		std::vector<float> getCOLDLenAngleSeperately(std::vector<linevec>& veclist,int model);
		
		/** build COLD feature */
		std::vector<float> getCOLDLogPolar(std::vector<linevec>& veclist);
		
		float scale;
		int nrad;
		int nang;
		
		bool silence;
};

/** an example to call COLD feature */
/*int main(int argc,char*argv[])
{
	if(argc!=8)
	{
		std::cout<<"Usage: "<<argv[0]<<" ppm scale kLength \n";
		return 1;
	}
	
	PamImage* im = new PamImage(argv[1]);
	
	if(im->getImageType()!=2)
		im=im->convert(2);
	
	dplBinaryMethod bin(*im,false);
	PamImage* otsu=bin.run(0);

	float scale = atof(argv[2]);
	
	int nrad = 7;
	int nang = 12;
	
	int kLength = atoi(argv[3]);
	
	
	COLDfeat cold(scale,nrad,nang);
	
	std::vector<float> feat = cold.getCOLDfeat(otsu,kLength);
	std::cout<<"feat size: "<<feat.size()<<"\n";
	
	delete im;
	delete otsu;
	return 1;
}*/


#endif
